@@ -1,7 +1,7 @@
 import asyncio
 import json
+import logging
 import time
-import warnings
 from asyncio import sleep
 from typing import Optional, Iterable
 
@@ -22,12 +22,15 @@ class PubSubClient:
     NONCE_REQUEST_VALUE = 'NONCE'
     LISTEN_REQUEST_KEY = 'LISTEN'
     PING_SEND_INTERVAL = 60 * 4.6
+    RECONNECT_PONG_TIMEOUT = 10
 
     def __init__(self):
         self.socket: Optional[websockets.client.WebSocketClientProtocol] = None
         self.listen_count = 0
         self._last_ping_sent_time = time.time()
+        self._waiting_for_pong = False
         self._pong_received = False
+        self._previously_sent_listen_data = set()
 
     @property
     def connected(self):
@@ -55,6 +58,22 @@ class PubSubClient:
 
         return json.dumps(data)
 
+    def _mark_pong_received(self):
+        self._pong_received = True
+        self._waiting_for_pong = False
+
+    def _mark_ping_sent(self):
+        self._pong_received = False
+        self._waiting_for_pong = True
+        self._last_ping_sent_time = time.time()
+
+    def _check_needs_reconnect(self):
+        return not self.connected or (
+                self._waiting_for_pong
+                and not self._pong_received
+                and self.last_ping_time_diff > self.RECONNECT_PONG_TIMEOUT
+        )
+
     async def listen_to_channel(self, channel_name: str, topics: Iterable[str], access_token: str = '', nonce=None) -> bool:
         if not self.connected:
             self.start_loop()
@@ -70,17 +89,18 @@ class PubSubClient:
 
         user_id = await get_user_id(channel_name)
         if user_id == -1:
-            warnings.warn(f'[PUBSUB-CLIENT] unable to get user id in pubsub client for channel "{channel_name}"')
+            logging.warning(f'[PUBSUB-CLIENT] unable to get user id in pubsub client for channel "{channel_name}"')
             return False
 
         topics = [f'{topic}{user_id}' for topic in topics]
         if not topics:
             return False
 
-        await self.socket.send(
-            self.create_listen_request_data(topics=topics, access_token=access_token, nonce=nonce or channel_name)
-        )
+        listen_data = self.create_listen_request_data(topics=topics, access_token=access_token, nonce=nonce or channel_name).strip()
+        if listen_data not in self._previously_sent_listen_data:
+            self._previously_sent_listen_data.add(listen_data)
 
+        await self.socket.send(listen_data)
         return True
 
     @property
@@ -93,7 +113,7 @@ class PubSubClient:
 
     async def _send_ping(self):
         await self.socket.send(json.dumps({'type': 'PING'}))
-        self._last_ping_sent_time = time.time()
+        self._mark_ping_sent()
 
     async def read(self, timeout: float = 10) -> Optional[str]:
         try:
@@ -115,21 +135,66 @@ class PubSubClient:
         if not task_exist(self.TASK_NAME):
             add_task(self.TASK_NAME, self._processor_loop())
 
+    async def _reconnect(self, listen_resend_interval=.7, reconnect_interval=5) -> bool:
+        import socket
+        try:
+            await self._connect()
+            # make sure connection is actually open
+            if not self.socket and not self.socket.open:
+                raise ValueError
+
+            # resend pubsub listen request
+            for listen_data in self._previously_sent_listen_data:
+                await self.socket.send(listen_data)
+                await asyncio.sleep(listen_resend_interval)
+
+            # signal reconnect was successful
+            return True
+        except (ValueError, socket.gaierror):
+            await asyncio.sleep(reconnect_interval)
+
+        # signal reconnect was unsuccessful
+        return False
+
     async def _processor_loop(self):
         while True:
             if self.socket is not None:
+                # keep reconnect logic behind the socket is not None check to be sure we had a previous connection
+                if self._check_needs_reconnect():
+                    await self._reconnect_loop()
+
                 try:
                     await self._read_and_handle()
                 except (json.JSONDecodeError, TypeError):
                     pass
+                except websockets.exceptions.ConnectionClosedError:
+                    # allow reconnect logic to try to re-establish a connection
+                    continue
 
                 await self._send_ping_if_needed()
+                if self._check_needs_reconnect():
+                    await self.socket.close()
             else:
                 await sleep(2)
 
+    async def _reconnect_loop(self):
+        backoff = 1
+        while True:
+            logging.warning(f'[PUBSUB_CLIENT] attempting reconnect...')
+            if await self._reconnect():
+                break
+
+            logging.warning(f'[PUBSUB_CLIENT] reconnect failed, retrying in {backoff}')
+            await asyncio.sleep(backoff)
+            backoff <<= 1
+
     async def _read_and_handle(self):
         raw_resp = await self.read(timeout=10)
         data = PubSubData(json.loads(raw_resp))
+
+        if data.is_pong:
+            self._mark_pong_received()
+
         await self._trigger_events(data)
 
     async def _trigger_events(self, data: 'PubSubData'):
